#ifndef __NOUVEAU_DEBUG_H__
#define __NOUVEAU_DEBUG_H__

#define NV_DBG_FATAL    0
#define NV_DBG_ERROR    1
#define NV_DBG_WARN     2
#define NV_DBG_INFO     3
#define NV_DBG_DEBUG    4
#define NV_DBG_TRACE    5
#define NV_DBG_PARANOIA 6
#define NV_DBG_SPAM     7

#endif
